import boto3
from s3imageservice.conf import settings
from storages.backends.s3boto3 import S3Boto3Storage


def get_bucket():
    session = boto3.session.Session(
        region_name=settings.AWS_REGION,
        aws_access_key_id=settings.AWS_ACCESS_KEY_ID,
        aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY
    )
    s3 = session.resource('s3')

    return s3.Bucket(settings.AWS_S3_BUCKET)


class Defaults(object):
    CACHE_HEADER = str('max-age=5')
    ROOT_PATH = 'elections'
    ACL = 'public-read'


defaults = Defaults


class StorageService(S3Boto3Storage):
    bucket_name = settings.AWS_S3_BUCKET
    access_key = settings.AWS_ACCESS_KEY_ID
    secret_key = settings.AWS_SECRET_ACCESS_KEY
    file_overwrite = True
    querystring_auth = False
    object_parameters = {
        'CacheControl': 'max-age=86400',
        'ACL': 'public-read',
    }
    location = settings.S3_UPLOAD_ROOT



from django.conf import settings as project_settings
from django.utils.decorators import method_decorator

from s3imageservice.conf import settings

from .importers import import_class


def secure(view):
    """
    Authentication decorator for views.
    If DEBUG is on, we serve the view without authenticating.
    Default is 'django.contrib.auth.decorators.login_required'.
    Can also be 'django.contrib.admin.views.decorators.staff_member_required'
    or a custom decorator.
    """
    auth_decorator = import_class(settings.AUTH_DECORATOR)
    return (
        view
        if project_settings.DEBUG
        else method_decorator(auth_decorator, name="dispatch")(view)
    )



from s3imageservice.models import Upload
from rest_framework import serializers


class UploadSerializer(serializers.ModelSerializer):
    class Meta:
        model = Upload
        fields = "__all__"



from s3imageservice.cms.views.base import CMSBaseView
from django.views.generic.base import RedirectView
from django.urls import reverse
from s3imageservice.models import Upload


class Detail(CMSBaseView):
    template_name = "detail.html"
    model = Upload

    def test_model_instance_exists(self, request, *args, **kwargs):
        Upload.objects.get(id=kwargs["upload"])

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context



"""
Use this file to configure pluggable app settings and resolve defaults
with any overrides set in project settings.
"""

from django.conf import settings as project_settings
from s3imageservice.utils.importers import import_class


class Settings:
    pass


Settings.AUTH_DECORATOR = getattr(
    project_settings,
    "S3IMAGESERVICE_AUTH_DECORATOR",
    "django.contrib.auth.decorators.login_required",
)

Settings.SECRET_KEY = getattr(
    project_settings, "S3IMAGESERVICE_SECRET_KEY", "a-bad-secret-key"
)

Settings.AWS_ACCESS_KEY_ID = getattr(
    project_settings, "S3IMAGESERVICE_AWS_ACCESS_KEY_ID", None
)

Settings.AWS_SECRET_ACCESS_KEY = getattr(
    project_settings, "S3IMAGESERVICE_AWS_SECRET_ACCESS_KEY", None
)

Settings.AWS_REGION = getattr(
    project_settings, "S3IMAGESERVICE_AWS_REGION", None
)

Settings.AWS_S3_BUCKET = getattr(
    project_settings, "S3IMAGESERVICE_AWS_S3_BUCKET", None
)

Settings.S3_UPLOAD_ROOT = getattr(
    project_settings, "S3IMAGESERVICE_S3_UPLOAD_ROOT", "uploads/s3imageservice"
)

Settings.AWS_S3_STATIC_ROOT = getattr(
    project_settings, "S3IMAGESERVICE_AWS_S3_STATIC_ROOT", None
)

Settings.API_AUTHENTICATION_CLASS = import_class(
    getattr(
        project_settings,
        "S3IMAGESERVICE_API_AUTHENTICATION_CLASS",
        "rest_framework.authentication.BasicAuthentication",
    )
)

Settings.API_PERMISSION_CLASS = import_class(
    getattr(
        project_settings,
        "S3IMAGESERVICE_API_PERMISSION_CLASS",
        "rest_framework.permissions.IsAuthenticated",
    )
)

Settings.MEDIA_ROOT = getattr(project_settings, "MEDIA_ROOT", "")

Settings.MEDIA_PATH = getattr(
    project_settings, "S3IMAGESERVICE_MEDIA_PATH", ""
)

Settings.FILE_MB_LIMIT = getattr(
    project_settings, "S3IMAGESERVICE_FILE_MB_LIMIT", 100
)

Settings.CMS_PAGE_SIZE = getattr(
    project_settings, "S3IMAGESERVICE_CMS_PAGE_SIZE", 50
)

Settings.CMS_DEFAULT_SIZES = getattr(
    project_settings, "S3IMAGESERVICE_CMS_DEFAULT_SIZES", []
)


settings = Settings



import uuid
from django.contrib.auth.models import User
from django.contrib.postgres.fields import JSONField
from django.db import models


class Upload(models.Model):
    """
    An image upload.
    """

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    timestamp = models.DateTimeField(auto_now_add=True)
    owner = models.ForeignKey(
        User, null=True, blank=True, on_delete=models.PROTECT
    )
    canonical = models.URLField(unique=True)
    data = JSONField(blank=True, null=True)
    slug = models.CharField(unique=True, max_length=50)

    def __str__(self):
        return self.canonical

    class Meta:
        ordering = ["-timestamp"]



from django.urls import path, include

from .views import ImageService

app_name = "s3imageservice"

urlpatterns = [
    path("", ImageService.as_view(), name="upload"),
    path("cms/", include("s3imageservice.cms.urls", namespace="cms")),
]
